import { Environment } from '@webprovisions/platform';
import { queryLinkTargets, queryPreRenderedTargets } from './target-queries';
import TriggerLinkMonitor from './trigger-link-monitor';
import distinct from './distinct';
export function attach(widget) {
    var triggerElements = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        triggerElements[_i - 1] = arguments[_i];
    }
    var widgetContainer = widget.container;
    var controller = widgetContainer.get('$controller');
    return controller.initialize().then(function () {
        // get the widget's existing instance of TriggerLinkMonitor.
        var monitor = widgetContainer.get('triggerLinkMonitor');
        // add all found elements
        monitor.watch(triggerElements);
    });
}
export function scan(implementation, rootElement) {
    if (rootElement === void 0) { rootElement = document; }
    // scan 'rootElement' for trigger elements matching the bindings
    var bindings = implementation.bindings;
    var triggerElementMap = [].concat(queryLinkTargets(rootElement, bindings), queryPreRenderedTargets(rootElement, bindings));
    // iterate each unique widget name and initialize the corresponding controller
    var uniqueWidgetNames = distinct(triggerElementMap.map(function (_a) {
        var widgetName = _a.widgetName;
        return widgetName;
    }));
    var promises = uniqueWidgetNames.map(function (widgetName) {
        var widget = implementation.getWidget(widgetName);
        var triggerElements = triggerElementMap
            .filter(function (item) { return item.widgetName === widgetName; })
            .map(function (item) { return item.element; });
        return attach.apply(void 0, [widget].concat(triggerElements)).then(function () { return widget; });
    });
    return Promise.all(promises);
}
function domContentLoaded() {
    return new Promise(function (resolve) {
        if (document.readyState === 'loading') {
            document.addEventListener('DOMContentLoaded', function () { return resolve(); });
        }
        else {
            resolve();
        }
    });
}
export default function bootstrapSingleImplementation(config, configureHandlers, environmentName, rootElement, configurationApiExtensions) {
    if (configureHandlers === void 0) { configureHandlers = []; }
    var environment = Environment.create({ configurationApiExtensions: configurationApiExtensions, name: environmentName });
    var implementation = environment.createImplementation(config);
    implementation.configure(function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        configureHandlers.forEach(function (configureHandler) { return configureHandler.apply(void 0, args); });
    });
    environment.events.subscribe('widget:pre-config', function (e, data) {
        /**
         * This event is triggered after pre-configuration is applied and before the widget
         * instance is created. Create a TriggerLinkMonitor for each widget.
         */
        var widget = e.target;
        var monitor = widget.container.get('triggerLinkMonitor');
        if (!monitor) {
            // create and register if not available
            monitor = new TriggerLinkMonitor(widget);
            widget.container.registerValue('triggerLinkMonitor', monitor);
        }
    });
    domContentLoaded()
        .then(function () { return environment.start(); })
        .then(function () { return scan(implementation, rootElement); });
    // environment.scan = () => {
    //   throw new Error('Not implemented');
    // };
    // environment.attach = () => {
    //   throw new Error('Not implemented');
    // };
    return environment;
}
//# sourceMappingURL=bootstrap-single-implementation.js.map